use Getopt::Long;
use strict;

my ( $test_case, $client_folder, $mtr_build_thread, $server_shutdown );

# Test case A: start master/slave servers, but do not start replication
# Execute the binlog
# Check the SELECT output on the master server to show the order

# Test case B: start master/slave servers, start replication
# Execute the binlog
# Check the SELECT output on the master server to show that the order is different from test case A

# Test case C: start master/slave servers, start replication
# Execute the binlog
# Check the SELECT output on master and slave to show that the order differs


GetOptions (
	"test_case=s" => \$test_case,
	"test-case=s" => \$test_case,
	"client_folder=s" => \$client_folder,
	"client-folder=s" => \$client_folder,
	"mtr_build_thread=i" => \$mtr_build_thread,
	"mtr-build-thread=i" => \$mtr_build_thread,
	"shutdown=i" => \$server_shutdown,
	);

$client_folder ||= "../bin";
$mtr_build_thread ||= 300;
$test_case ||= 'C';
$server_shutdown = 1 unless defined $server_shutdown;

unless ( -e "${client_folder}/mysql" and -e "${client_folder}/mysqladmin" ) {
	print "ERROR: mysql and/or mysqladmin not found\n";
	print "Please use correct value for --client_foder option\n\n";
	exit 1;
}

$ENV{MASTER_MYPORT}=$mtr_build_thread*10 + 10000;
$ENV{SLAVE_MYPORT}=$ENV{MASTER_MYPORT} + 1;

my ( $master, $slave ) = ( "", "" );

my $pid = fork();
die "Could not fork" unless defined $pid;

if ( ! $pid ) {
	readpipe( "perl mysql-test-run.pl --start --mtr-build-thread=$mtr_build_thread --mysqld=--max_connections=120 --mysqld=--innodb --mysqld=--binlog_format=mixed  rpl_alter 2>&1" );
	print "Server is not running anymore \n";
	exit;
}
else {
	while ( system( "${client_folder}/mysqladmin -uroot --protocol=tcp -s --port=$ENV{MASTER_MYPORT} ping" ) ) { sleep 1 };

	unless ( $test_case eq 'A' ) {
		system( "${client_folder}/mysql -uroot --protocol=tcp --port=$ENV{SLAVE_MYPORT} -e \"stop slave; change master to master_port=$ENV{MASTER_MYPORT}, master_host=\'127.0.0.1\', master_user=\'root\'; start slave;\"" );
	}
	system( "${client_folder}/mysql -uroot --protocol=tcp --port=$ENV{MASTER_MYPORT} < ./rand_test/binlog_init.out" );
	open( MASTER, "${client_folder}/mysql -uroot --protocol=tcp --port=$ENV{MASTER_MYPORT} --table -e \""
		. "SET \@\@RAND_SEED1=737571346, \@\@RAND_SEED2=26080376; "
		. "SELECT user_num, name, location FROM betony_features.t_part_to_seconds ORDER BY RAND() LIMIT 4;\" |" );
	while (<MASTER>) {
		$master .= $_;
	}
	close( MASTER );
	print "Master:\n$master\n";

	if ( $test_case eq 'C' ) {
		sleep(3);
		open( SLAVE, "${client_folder}/mysql -uroot --protocol=tcp --port=$ENV{SLAVE_MYPORT} --table -e \""
			. "SET \@\@RAND_SEED1=737571346, \@\@RAND_SEED2=26080376; "
			. "SELECT user_num, name, location FROM betony_features.t_part_to_seconds ORDER BY RAND() LIMIT 4;\" |" );
		while (<SLAVE>) {
			$slave .= $_;
		}
		close( SLAVE );
		print "Slave:\n$slave\n";
		if ( $master and $slave and ( $master ne $slave ) ) {
			print "Check the difference between master and slave output\n\n";
		}
		else {
			print "Test did not reveal the difference between master and slave\n";
		}
	}	
	else {
		print "Check the order, compare it to the output of test case " . ( $test_case eq 'A' ? 'B' : 'A' ) . "\n\n";
	}
}

if ( $server_shutdown ) {
	print "Stopping servers...\n";
	system( "${client_folder}/mysqladmin -uroot --protocol=tcp --port=$ENV{MASTER_MYPORT} shutdown" );
	until ( system( "${client_folder}/mysqladmin -uroot --protocol=tcp --silent --port=$ENV{MASTER_MYPORT} ping > /dev/null" ) ) { sleep 1 };
	until ( system( "${client_folder}/mysqladmin -uroot --protocol=tcp --silent --port=$ENV{SLAVE_MYPORT} ping > /dev/null" ) ) { sleep 1 };
}
